@@ -62,15 +62,19 @@ import java.net.HttpURLConnection;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLEncoder;
+import java.nio.charset.StandardCharsets;
 import java.util.List;
 import java.util.Map;
-import java.util.Scanner;
 import java.util.Map.Entry;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import java.util.Scanner;
 import javax.management.openmbean.CompositeData;
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.SSLException;
 import org.json.JSONException;
 import org.json.JSONObject;
+import com.openexchange.java.Streams;
 import com.openexchange.report.client.configuration.ReportConfiguration;
 import com.openexchange.report.client.container.ClientLoginCount;
 import com.openexchange.report.client.container.ContextDetail;
@@ -89,7 +93,7 @@ public class TransportHandler {
 
     private static final String REPORT_SERVER_URL = "activation.open-xchange.com";
 
-    private static final String REPORT_SERVER_CLIENT_AUTHENTICATION_STRING = "BKvKJhhjBuz6fUGvKGjGhgCkjvchgvjgaypHFGc7tvjVc76Vja";
+    private static final String REPORT_SERVER_CLIENT_AUTHENTICATION_STRING = "rhadsIsAgTicOpyodNainPacloykAuWyribZydkarbEncherc4";
 
     private static final String POST_CLIENT_AUTHENTICATION_STRING_KEY = "clientauthenticationstring";
 
@@ -101,13 +105,13 @@ public class TransportHandler {
 
     public TransportHandler() {}
 
-    public void sendReport(final List<Total> totals, final List<MacDetail> macDetails, final List<ContextDetail> contextDetails, Map<String, String> serverConfiguration, final String[] versions, final ClientLoginCount clc, final ClientLoginCount clcYear, final boolean savereport) throws IOException, JSONException {
+    public void sendReport(final List<Total> totals, final List<MacDetail> macDetails, final List<ContextDetail> contextDetails, Map<String, String> serverConfiguration, final String[] versions, final ClientLoginCount clc, final ClientLoginCount clcYear, final boolean savereport) throws Exception {
         final JSONObject metadata = buildJSONObject(totals, macDetails, contextDetails, serverConfiguration, versions, clc, clcYear);
 
         send(metadata, savereport);
     }
 
-    private void send(JSONObject metadata, boolean savereport) throws IOException, JSONException {
+    private void send(JSONObject metadata, boolean savereport) throws Exception {
         final ReportConfiguration reportConfiguration = new ReportConfiguration();
 
         try {
@@ -134,7 +138,7 @@ public class TransportHandler {
      * Try to send the report to the saved REPORT_SERVER_URL over https, if the given parameter is set. If
      * https fails, try it with http. If that fails, save the report to hard-drive and save location
      * to logfile.
-     * 
+     *
      * @param isHttps
      * @param reportConfiguration
      * @param report
@@ -166,12 +170,13 @@ public class TransportHandler {
         httpURLConnection.setReadTimeout(2500);
         httpURLConnection.setUseCaches(false);
         httpURLConnection.setDoOutput(true);
+        httpURLConnection.setFixedLengthStreamingMode(report.length()); // This enforces 'sun.net.www.protocol.http.HttpURLConnection' to use a "streaming" output stream; otherwise outgoing data is stored into a ByteArrayOutputStream (likely causing an OOME; see Bug 57260)
         httpURLConnection.setDoInput(true);
         httpURLConnection.setRequestMethod("POST");
         httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
 
         if ("true".equals(reportConfiguration.getUseProxy().trim()) && "true".equals(reportConfiguration.getProxyAuthRequired().trim())) {
-            final String proxyAutorizationProperty = "Basic " + Base64.encode((reportConfiguration.getProxyUsername().trim() + ":" + reportConfiguration.getProxyPassword().trim()).getBytes());
+            final String proxyAutorizationProperty = "Basic " + Base64.encode((reportConfiguration.getProxyUsername().trim() + ":" + reportConfiguration.getProxyPassword().trim()).getBytes(StandardCharsets.US_ASCII));
 
             Authenticator.setDefault(new ProxyAuthenticator(reportConfiguration.getProxyUsername().trim(), reportConfiguration.getProxyPassword().trim()));
 
@@ -193,24 +198,34 @@ public class TransportHandler {
                 System.out.println("Trying to send without ssl.");
                 sendReport(false, reportConfiguration, metadata);
                 return;
-            } else
+            } else {
                 saveReportToHardDrive(metadata);
+            }
         }
 
         if (httpURLConnection.getResponseCode() != HttpURLConnection.HTTP_OK) {
             System.err.println("Report sending failure, unable to send via http...");
-            if (isHttps)
+            if (isHttps) {
                 sendReport(false, reportConfiguration, metadata);
-            else
+            } else {
                 saveReportToHardDrive(metadata);
+            }
             throw new MalformedURLException("Problem contacting report server: " + httpURLConnection.getResponseCode());
         }
-        final BufferedReader in = new BufferedReader(new InputStreamReader(httpURLConnection.getInputStream()));
-        String buffer = "";
-        while ((buffer = in.readLine()) != null) {
-            System.out.println(new StringBuilder().append(REPORT_SERVER_URL).append(" said: ").append(buffer).toString());
+        String charset = httpURLConnection.getContentType();
+        if (null != charset) {
+            Pattern charsetPattern = Pattern.compile("charset= *([a-zA-Z-0-9_]+)(;|$)");
+            Matcher m = charsetPattern.matcher(charset);
+            charset = m.find() ? m.group(1) : null;
+        }
+        try (BufferedReader in = new BufferedReader(new InputStreamReader(httpURLConnection.getInputStream(), null == charset ? "UTF-8" : charset));) {
+            StringBuilder sb = new StringBuilder(REPORT_SERVER_URL).append(" said: ");
+            int reslen = sb.length();
+            for (String line; (line = in.readLine()) != null;) {
+                sb.setLength(reslen);
+                System.out.println(sb.append(line).toString());
+            }
         }
-        in.close();
     }
 
     private String createReportString(JSONObject metadata) throws JSONException {
@@ -254,7 +269,7 @@ public class TransportHandler {
 
     /**
      * Save the given report to hard-drive and print the location into the logfile.
-     * 
+     *
      * @param report
      * @throws IOException
      * @throws JSONException
@@ -265,11 +280,14 @@ public class TransportHandler {
         System.out.println("Saving report to " + tmpfile.getAbsolutePath());
         // Compose report from stored data, depending on the report-type
         DataOutputStream tfo = new DataOutputStream(new FileOutputStream(tmpfile));
-        tfo.writeBytes(reportString);
-        if (metadata.getBoolean("needsComposition")) {
-            appendStoredContentToOutputstream(metadata, tfo, false);
+        try {
+            tfo.writeBytes(reportString);
+            if (metadata.getBoolean("needsComposition")) {
+                appendStoredContentToOutputstream(metadata, tfo, false);
+            }
+        } finally {
+            Streams.close(tfo);
         }
-        tfo.close();
     }
 
     private JSONObject buildJSONObject(final List<Total> totals, final List<MacDetail> macDetails, final List<ContextDetail> contextDetails, Map<String, String> serverConfiguration, final String[] versions, final ClientLoginCount clc, final ClientLoginCount clcYear) throws JSONException {
@@ -360,7 +378,7 @@ public class TransportHandler {
         return retval;
     }
 
-    public void sendASReport(CompositeData report, boolean savereport) throws IOException, JSONException {
+    public void sendASReport(CompositeData report, boolean savereport) throws Exception {
         send(new JSONObject((String) report.get("data")), savereport);
     }
 }
